package com.ruoyi.common.security.config;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.interceptor.SaInterceptor;
import cn.dev33.satoken.router.SaRouter;
import cn.dev33.satoken.stp.StpUtil;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.utils.SpringUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.security.utils.LoginHelper;
import com.ruoyi.common.security.config.properties.SecurityProperties;
import com.ruoyi.common.security.handler.AllUrlHandler;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.AutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.util.Arrays;

/**
 * 权限安全配置
 *
 * @author Lion Li
 */

@Slf4j
@AutoConfiguration
@EnableConfigurationProperties(SecurityProperties.class)
@RequiredArgsConstructor
public class SecurityConfig implements WebMvcConfigurer {

    private final SecurityProperties securityProperties;

    /**
     * 注册 Sa-Token 路由拦截器
     */
    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        // 注册路由拦截器，自定义验证规则
        registry.addInterceptor(new SaInterceptor(handler -> {
                //AllUrlHandler allUrlHandler = SpringUtils.getBean(AllUrlHandler.class);
                // 登录验证 -- 排除多个路径
                SaRouter
                    // 获取所有的
                    //.match(allUrlHandler.getUrls())  // 拦截的 path 列表
                    .match("/**")
                    .notMatch(
                        Arrays.asList(
                                "/*.html",
                                "/**/*.html",
                                "/**/*.css",
                                "/**/*.js",
                                "/**/*.css",
                                "/**/*.js",
                                "/profile/**",
                                "/favicon.ico",
                                "/error",
                                "/*/api-docs",
                                "/*/api-docs/**",
                                "/actuator",
                                "/actuator/**",
                                "/login",
                                "/register",
                                "/captchaImage",
                                "/captcha/get",
                                "/captcha/check"))
                    .check(() -> {
                        // 检查是否登录 是否有token
                        StpUtil.checkLogin();

                        //TODO :以后完善多平台登录校验clientID功能

                    });
            })).addPathPatterns("/**");
            // 排除不需要拦截的路径
            //.excludePathPatterns(securityProperties.getExcludes());

    }
}
